/*!******************************************************************/
/*  File: ig27.c                                                    */
/*  ============                                                    */
/*                                                                  */
/*  This file includes:                                             */
/*                                                                  */
/*  igintt();     Inits serial port                                 */
/*  igextt();     Resets serial port                                */
/*  igsini();     Inits signals                                     */
/*  sigtrp();     Trap-routine for signals                          */
/*  v3fmov();     move/rename file                                  */
/*  v3fcpy();     Copy files                                        */
/*  v3fapp();     Add two files                                     */
/*  v3fdel();     Delete file                                       */
/*  v3facc();     Test file accessability                           */
/*  v3ftst();     Test file existance                               */
/*  v3mkdr();     Create directory                                  */
/*  igcmos();     Interactive OS-command                            */
/*  v3cmpw();     Compare strings with wildcard                     */
/*                                                                  */
/*  This file is part of the VARKON IG Library.                     */
/*  URL:  http://www.varkon.com                                     */
/*                                                                  */
/*  This library is free software; you can redistribute it and/or   */
/*  modify it under the terms of the GNU Library General Public     */
/*  License as published by the Free Software Foundation; either    */
/*  version 2 of the License, or (at your option) any later         */
/*  version.                                                        */
/*                                                                  */
/*  This library is distributed in the hope that it will be         */
/*  useful, but WITHOUT ANY WARRANTY; without even the implied      */
/*  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR         */
/*  PURPOSE.  See the GNU Library General Public License for more   */
/*  details.                                                        */
/*                                                                  */
/*  You should have received a copy of the GNU Library General      */
/*  Public License along with this library; if not, write to the    */
/*  Free Software Foundation, Inc., 675 Mass Ave, Cambridge,        */
/*  MA 02139, USA.                                                  */
/*                                                                  */
/*  (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se */
/*                                                                  */
/********************************************************************/

#include "../../DB/include/DB.h"
#include "../include/IG.h"
#include "../include/screen.h"
#include "../../EX/include/EX.h"
#include "../../GP/include/GP.h"

#include <signal.h>

#ifdef UNIX
#include <sys/ioctl.h>
/*#include <sys/termios.h>*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif

#ifdef WIN32
#include <direct.h>
#include <io.h>
#endif

#ifdef UNIX
/* static struct termios trmpar; */
extern MNUALT smbind[];

/*     trmpar innehller terminalport-parametrar.
       smbind innehller bla. snabbvalen fr Quit och
       interrupt.
*/
#endif

extern short  igtrty,v3mode,ialy,rmarg,bmarg;
extern bool   intrup;

static void sigtrp(int sigval);

/*!******************************************************/

        short igintt()

/*      Stter upp terminalporten.
 *
 *      In: Inget.
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 20/2/85 J. Kjellander
 *
 *      12/10/88 Tagit bort setbuf(), J. Kjellander
 *      1999-04-07 termios, J.Kjellander
 *
 ******************************************************!*/

  {
/*
#ifdef UNIX
    struct termios tmp;
#endif
*/

/*
***Det hr skippar vi p en del datorer.
*/
#ifdef WIN32
    return(0);
#endif
/*
***Ls nuvarande terminalparametrar och spara i trmpar.
***Kopiera till tmp och stt !ECHO samt !ICANON.
*
#ifdef UNIX
    if ( igtrty == VT100 )
      {
      igflsh();
      tcgetattr(0,&trmpar);
      V3MOME(&trmpar,&tmp,sizeof(struct termios));
      tmp.c_lflag &= ~ICANON;
      tmp.c_lflag &= ~ECHO;
      tcsetattr(0,TCSANOW,&tmp);
      }
#endif
*
***End.
*/
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short igextt()

/*      terstller terminalporten i ursprungligt skick.
 *
 *      In: Inget.
 *
 *      Ut: Inget.
 *
 *      (C)microform ab 16/6/85 J. Kjellander
 *
 *      1999-04-07 termios, J.Kjellander
 *
 ******************************************************!*/

  {

/*
***Det hr skippar vi p en del datorer.
*/
#ifdef WIN32
    return(0);
#endif
/*
***Skriv tillbaks trmpar.
*
#ifdef UNIX
    if ( igtrty == VT100 )
      {
      igflsh();
      tcsetattr(0,TCSANOW,&trmpar);
      }
#endif
*
***End.
*/
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short igsini()

/*      Stter upp vilka signaler som skall fngas.
 *
 *      In: Inget.
 *
 *      Ut: Inget.
 *
 *      (C)microform ab 25/11/85 J. Kjellander
 *
 *      23/10/86 Ny hantering av SIGFPE, J. Kjellander
 *      4/3/92   Hangup, J. Kjellander
 *      3/4/92   SIGTERM, J. Kjellander
 *
 ******************************************************!*/

  {

#ifdef WIN32
   return(0);
#else
/*
***Fnga signalerna hangup, interrupt och quit.
*/
    signal(SIGHUP,sigtrp);
    signal(SIGINT,sigtrp);
    signal(SIGQUIT,sigtrp);
    signal(SIGTERM,sigtrp);
/*
***Fnga inte flyttalsfel men lt core-fil skapas.
*/
    signal(SIGFPE,SIG_DFL);
#endif

    return(0);
  }

/********************************************************/
/*!******************************************************/

static void sigtrp(int sigval)

/*      Trap-rutin fr signaler.
 *
 *      In: sigval => Nummer p signal som orsakat trap.
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 20/2/85 J. Kjellander
 *
 *      9/5/86   SIGFPE, J. Kjellander
 *      23/10/86 Ny hantering av SIGFPE, J. Kjellander
 *      16/3/88  v3exit(), J. Kjellander
 *      4/3/92   Hangup, J. Kjellander
 *      3/4/92   SIGTERM, J. Kjellander
 *      1998-02-22 void, J.Kjellander
 *
 ******************************************************!*/

  {

#ifdef WIN32
   return;
#else

   switch (sigval)
     {
/*
***Hangup, tex. om ett modem kopplar ner.
*/
     case SIGHUP:
     case SIGTERM:
     signal(SIGHUP,SIG_IGN);
     signal(SIGTERM,SIG_IGN);
     if ( v3mode & (BAS_MOD+RIT_MOD) ) igexsa();
     v3exit();
     break;
/*
***Interrupt, normalt <CTL>c.
*/
     case SIGINT:
     igsini();
     intrup = TRUE;
     break;
/*
***Quit, normalt <DEL>.
*/
     case SIGQUIT:
     if ( v3mode & (BAS_MOD+RIT_MOD) ) igexsa();
     v3exit();
     break;
     }
#endif

   return;
  }

/********************************************************/
/*!******************************************************/

        short v3fmov(
        char *from,
        char *to)

/*      ndrar filen "from:s" namn till "to".
 *
 *      In: from => Pekare till gammal vgbeskrivning.
 *          to   => Pekare till ny vgbeskrivning.
 *
 *      Ut: Inget.
 *
 *      FV:      0 => Ok.
 *          IG0482 => Kan ej byta namn p filen %s
 *          IG0512 => Kan ej skapa filen %s
 *          IG0632 => MOVE_FILE, %s r ppen.
 *
 *      (C)microform ab 1996-01-25 J. Kjellander
 *
 ******************************************************!*/

  {
#ifdef SCO_UNIX
    char buf[1000];
#endif
/*
***Kolla att ingen av filerna r ppnade frn MBS.
*/
   if ( v3isop(from) ) return(erpush("IG0632",from));
   if ( v3isop(to) )   return(erpush("IG0632",to));
/*
***I WIN32 fr inte en fil med samma namn finnas sedan tidigare.
*/
#ifdef WIN32
    if ( v3facc(to,'X') ) v3fdel(to);
#endif
/*
***Byt namn. C-rutinen rename() finns inte i Xenix.
*/
#ifdef SCO_UNIX
    sprintf(buf,"mv %s %s",from,to);
    system(buf);
#else
    if ( rename(from,to) == 0 ) return(0);
    else
      {
      erpush("IG0482",from);
      return(erpush("IG0512",to));
      }
#endif
/*
***Slut.
*/
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short v3fcpy(
        char *from,
        char *to)

/*      Kopierar filen "from" till filen "to".
 *
 *      In: from => Pekare till vgbeskrivning fr gamla filen.
 *          to   => Pekare till vgbeskrivning fr nya filen.
 *
 *      Ut: Inget.
 *
 *      FV:      0 => Ok.
 *          IG0073 => Kan ej ppna filen %s
 *          IG0063 => Fel vid skrivning till %s
 *          IG0622 => COPY_FILE, %s r ppen.
 *
 *      (C)microform ab 30/7/85 J. Kjellander
 *
 *      9/1/96  WIN32, J. Kjellander
 *
 ******************************************************!*/

  {
     FILE  *frompk;
     FILE  *topk;
     char   buf[BUFSIZ];
     size_t n;

/*
***Kolla att ingen av filerna r ppnade frn MBS.
*/
   if ( v3isop(from) ) return(erpush("IG0622",from));
   if ( v3isop(to) )   return(erpush("IG0622",to));
/*
***Prova att ppna den gamla filen.
*/
#ifdef WIN32
     if ( (frompk=fopen(from,"rb")) == 0 ) return(erpush("IG0073",from));
#else
     if ( (frompk=fopen(from,"r")) == 0 ) return(erpush("IG0073",from));
#endif
/*
***ppna den nya filen.
*/
#ifdef WIN32
     if ( (topk=fopen(to,"wb")) == 0 ) return(erpush("IG0073",to));
#else
     if ( (topk=fopen(to,"w")) == 0 ) return(erpush("IG0073",to));
#endif
/*
***Kopiera.
*/
loop:
     n = fread(buf,1,BUFSIZ,frompk);

     if ( n != 0 )
       {
       if ( fwrite(buf,1,n,topk) == n ) goto loop;
       else return(erpush("IG0063",from));
       }
/*
***Stng filerna.
*/
     fclose(frompk);
     fclose(topk);

     return(0);
  }

/********************************************************/
/*!******************************************************/

        short v3fapp(
        char *from,
        char *to)

/*      Lgger till filen "from" sist i filen "to".
 *
 *      In: from => Pekare till fil att addera till to.
 *          to   => Pekare till fil som det skall lggas till i.
 *
 *      Ut: Inget.
 *
 *      FV:      0 => Ok.
 *          IG0073 => Kan ej ppna filen %s
 *          IG0063 => Fel vid skrivning till %s
 *          IG0602 => APPEND_FILE, %s r ppen.
 *
 *      (C)microform ab 1996-01-25 J. Kjellander
 *
 ******************************************************!*/

  {
     FILE  *frompk;
     FILE  *topk;
     char   buf[BUFSIZ];
     size_t n;
/*
***Kolla att ingen av filerna r ppnade frn MBS.
*/
   if ( v3isop(from) ) return(erpush("IG0602",from));
   if ( v3isop(to) )   return(erpush("IG0602",to));
/*
***Prova att ppna den fil det skall lsas ur.
*/
#ifdef WIN32
     if ( (frompk=fopen(from,"rb")) == 0 ) return(erpush("IG0073",from));
#else
     if ( (frompk=fopen(from,"r")) == 0 ) return(erpush("IG0073",from));
#endif
/*
***ppna den fil som det skall skrivas (sist) i.
*/
#ifdef WIN32
     if ( (topk=fopen(to,"ab")) == 0 ) return(erpush("IG0073",to));
#else
     if ( (topk=fopen(to,"a")) == 0 ) return(erpush("IG0073",to));
#endif
/*
***Ls och skriv.
*/
loop:
     n = fread(buf,1,BUFSIZ,frompk);

     if ( n != 0 )
       {
       if ( fwrite(buf,1,n,topk) == n ) goto loop;
       else return(erpush("IG0063",from));
       }
/*
***Stng filerna.
*/
     fclose(frompk);
     fclose(topk);

     return(0);
  }

/********************************************************/
/*!******************************************************/

        short v3fdel(char *fil)

/*      Tar bort en fil.
 *
 *      In: fil => Pekare till vgbeskrivning fr filen.
 *
 *      Ut: Inget.
 *
 *      Felkod: IG0492 = Kan ej ta bort filen %s
 *              IG0612 => DELETE_FILE, %s r ppen.
 *
 *      (C)microform ab 29/11/88 J. Kjellander
 *
 ******************************************************!*/

  {
/*
***Kolla att inte filen r ppnad frn MBS.
*/
   if ( v3isop(fil) ) return(erpush("IG0612",fil));
/*
***Ta bort den.
*/
#ifdef UNIX
     if ( unlink(fil) == 0 ) return(0);
     else                    return(erpush("IG0492",fil));
#endif

#ifdef WIN32
     if ( unlink(fil) == 0 ) return(0);
     else                    return(erpush("IG0492",fil));
#endif

#ifdef VMS
     delete(fil);
     return(0);
#endif
  }

/********************************************************/
/*!******************************************************/

        bool v3facc(
        char *fil,
        char  mode)

/*      Kollar om fil kan accessas p visst stt.
 *
 *      In: fil  = Pekare till vgbeskrivning fr filen.
 *          mode = "R", "W" eller "X".
 *
 *      Ut: Inget.
 *
 *      FV: TRUE  = Filen kan accessas p det stt som nskats.
 *          FALSE = Det gr inte.
 *
 *      (C)microform ab 1996-01-26 J. Kjellander
 *
 ******************************************************!*/

  {
   int status,amode;
/*
***Vilken accessmode nskas ?
*/
   switch ( mode )
     {
#ifdef UNIX
     case 'r': case 'R': amode = R_OK; break;
     case 'w': case 'W': amode = W_OK; break;
     case 'x': case 'X': amode = F_OK; break;
#endif
#ifdef WIN32
     case 'r': case 'R': amode = 4; break;
     case 'w': case 'W': amode = 2; break;
     case 'x': case 'X': amode = 0; break;
#endif
     default: return(FALSE);
     }
/*
***Anropa access().
*/
   status = access(fil,amode);
/*
***Hur gick det ?
*/
   if ( status == 0 ) return(TRUE);
   else               return(FALSE);
  }

/********************************************************/
/*!******************************************************/

        bool v3ftst(char *fil)

/*      Kollar om fil gr att ppna fr lsning. OBS,
 *      i Win95 fr denna rutin inte anvndas p en
 *      filkatalog-fil!!!!!
 *
 *      In: fil => Pekare till vgbeskrivning fr filen.
 *
 *      Ut: Inget.
 *
 *      FV: TRUE  = Filen finns.
 *          FALSE = Filen finns ej eller gr ej att ppna.
 *
 *      (C)microform ab 27/11/88 J. Kjellander
 *
 ******************************************************!*/

  {
     FILE  *f;

/*
***Prova att ppna den gamla filen.
*/
     if ( (f=fopen(fil,"r")) == 0 ) return(FALSE);
     else
       {
       fclose(f);
       return(TRUE);
       }
  }

/********************************************************/
/*!******************************************************/

        short v3mkdr(char *dirnam)

/*      Skapar directory.
 *
 *      In: fil => Pekare till vgbeskrivning fr filkatalog.
 *
 *      Ut: Inget.
 *
 *      FV:   0 = Filkatalog skapad.
 *          < 0 = Kan ej skapa filkatalog.
 *
 *      Felkoder: IG0402 = Kan ej skapa filkatalog %s, fel frn OS !
 *
 *      (C)microform ab 22/9/95 J. Kjellander
 *
 *      1998-02-06 IXUSR etc. J.Kjellander
 *
 ******************************************************!*/

  {
#ifdef UNIX
    if ( mkdir(dirnam,S_IXUSR | S_IRUSR | S_IWUSR | S_IROTH |
                                 S_IXOTH | S_IRGRP | S_IXGRP) < 0 )
      {
      return(erpush("IG0402",dirnam));
      }
    else
      {
      return(0);
      }
#endif

#ifdef WIN32
    if ( mkdir(dirnam) < 0 ) return(erpush("IG0402",dirnam));
    else                     return(0);
#endif
  }

/********************************************************/
/*!******************************************************/

       short igcmos(char oscmd[])

/*      Interaktivt kommando till OS.
 *
 *      In: oscmd -> Kommandostrng
 *
 *      Ut: Inget.
 *
 *      FV: Inget.
 *
 *      (C)microform ab 5/3/88 J. Kjellander
 *
 *      4/11/88 CGI, N220G, J. Kjellander
 *
 ******************************************************!*/

 {
   char   s[2];
   short  status,oltrty;
   double tmpcng,tmprsk;
   VY     tmpavy;

/*
***Spara aktuell terminaltyp i oltrty. Under OS-kommandot
***stts terminalen tillflligt om till VT100 bla. fr att
***kunna lsa in svaret p frgan "Fortstt = <CR>".
*/
   oltrty = igtrty;

   switch ( igtrty )
     {
/*
***CGI, lmna CGI och gr skrmen till en VT100.
***ialy stts hr till 25 s att utskrifter frn UNIX inte
***skrivs ver (24 rader).
*/
     case CGI:
     gpgwin(&tmpavy);
     gpgrsk(&tmprsk);
     gpgtcn(&tmpcng);
     gpexit();
     igtrty = VT100;
     ialy = 25;
     break;
/*
***Tek4010-terminaler utan VT100-bildminne. Sudda skrmen
***och stll den i VT100-mode.
*/
     case N220G:
     case MSCOLOUR:
     case MSMONO:
     igpstr("\033[?38l",NORMAL);
     igtrty = VT100;
     break;
     }
/*
***vriga terminaler, rensa bara VT100-bildminnet och terstll
***porten i det lge den var d V3 startades.
*/
   igersc();
   igmvac(1,1);
   igextt();
/*
***Starta upp ny process och vnta tills den r klar.
***Oavsett terminaltyp har igintt() gjorts i v3.c, allts
***mste igextt gras nu.
*/
   EXos(oscmd,(short)0);
/*
***Vnta p anvndarens <CR>.
*/
   igintt();
   status = igssip(iggtts(3),s,"",1);
/*
***terstll skrm (och port).
*/
   igtrty = oltrty;

   switch ( igtrty )
     {
/*
***CGI, lmna VT-100 mode, initiera CGI och terstll
***skrmen utan att rita om grafiken.
*/
     case CGI:
     gpinit("CGI");
     gpswin(&tmpavy);
     gpstvi(&tmpavy);
     gpstcn(tmpcng);
     gpsrsk(tmprsk);
     gpdram();
     igupmu();
     iggnsa();
     break;
/*
***Tek-4010 utan VT100-bildminne. terg till Tek-mode,
***terstll skrmen utan att rita om grafiken.
*/
     case N220G:
     case MSCOLOUR:
     case MSMONO:
     igpstr("\033[?38h",NORMAL);
     gpdram();
     igupmu();
     iggnsa();
     break;
/*
***Skrmar med VT100-bildminne. terstll VT100 skrmen.
***Grafiken ligger kvar i det grafiska bildminnet.
*/
     default:
     igmvac(1,1);
     igerar(rmarg,bmarg);
     igupmu();
     iggnsa();
     break;
     }
/*
***Slut.
*/
   return(status);
 }

/*********************************************************************/
/*!******************************************************/

        bool  v3cmpw(
        char *wc_str,
        char *tststr)

/*      Jmfr 2 strngar varav en med wildcard "*".
 *      Godtyckligt antal stjrnor fr frekomma i 
 *      skstrngen (wc_str) varsomhelst men inte intill
 *      varandra. Om s returneras FALSE.
 *
 *      Om ngon av strngarna = "" returneras FALSE.
 *
 *      OBS ! Kopia av denna rutin finns i DBtraverse.c
 *      v3cmpw() anvnds fn. inte !
 *
 *      In: wc_str => Strng med 0, 1 eller flera stjrnor.
 *          tststr => Strng utan stjrnor.
 *
 *      Ut:  TRUE  = Strngarna matchar.
 *           FALSE = Matchar ej.
 *
 *      (C)microform ab 1996-02-12 J. Kjellander
 *
 *      1998-04-02 Bug, J.Kjellander
 *
 ******************************************************!*/

  {
   register char *p1,*p2;

/*
***Initiering.
*/
   p1 = wc_str;
   p2 = tststr;
/*
***Innan vi brjar jmfra mste vi kolla om ngon eller
***bda = "" eftersom resten av rutinen frutstter att
***s inte r fallet.
*/
   if ( *p1 == '\0'  ||  *p2 == '\0' ) return(FALSE);
/*
***Ingen av strngarna r "". Stt igng och jmfr!
*/
loop1:
/*
***r det en stjrna.
*/
   if ( *p1 == '*' )
     {
   ++p1;
/*
***Ja, skstrngen slutar med en stjrna.
*/
     if ( *p1 == '\0' ) return(TRUE);
/*
***2 stjrnor efter varandra r inte tilltet.
*/
     else if ( *p1 == '*' ) return(FALSE);
/*
***Stjrnan ej i slutet. Leta upp nsta tecken i p2 som matchar
***tecknet efter stjrnan.
*/
     else
       {
loop2:
/*
***Ett tecken i p2 matchar tecknet efter stjrnan i p1.
***Strippa eventuella ytterligare lika tecken i p2 som inte
***matchar nsta tecken i p1. Tex. p1=*poi och p2=me_ppoi
***1998-04-02 JK.
*/
       if ( *p1 == *p2 )
         {
         while ( *p1 == *(p2+1)  &&  *(p2+1) != *(p1+1) ) ++p2;
         goto loop1;
         }
       else
         {
       ++p2;
         if ( *p2 == '\0' ) return(FALSE);
         goto loop2;
         }
       }
     }
/*
***Ingen stjrna, jmfr 2 riktiga tecken.
*/
   else
     {
/*
***Lika.
*/
     if ( *p1 == *p2 )
       {
     ++p1;
     ++p2;
       if ( *p1 == '\0' )
         {
/*
***p1 och p2 bda slut.
*/
         if ( *p2 == '\0' ) return(TRUE);
/*
***p1 slut men inte p2.
*/
         else return(FALSE);
         }
       else
         {
/*
***p1 inte slut men p2.
*/
         if ( *p2 == '\0' )
           {
           if ( *p1 == '*'  &&  *(p1+1) == '\0' ) return(TRUE);
           else                                   return(FALSE);
           }
/*
***Varken p1 eller p2 slut.
*/
         else goto loop1;
         }
       }
/*
***Ej lika.
*/
     else return(FALSE);
     }
  }

/********************************************************/
