/**********************************************************************
*
*    eval2.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evsymb();      Evaluerar SYMB
*    evldim();      Evaluerar LDIM
*    evcdim();      Evaluerar CDIM
*    evrdim();      Evaluerar RDIM
*    evadim();      Evaluerar ADIM
*    evxht();       Evaluerar XHATCH
*    evarea();      Evaluerar AREA
*    evsuar();      Evaluerar SURFACE_AREA
*    evsear();      Evaluerar SECTION_AREA
*    evsecg();      Evaluerar SECTION_CGRAV
*    evcgra();      Evaluerar CGRAV
*    evtext();      Evaluerar TEXT
*    evbpln();      Evaluerar B_PLANE
*
*    evtrmv();      Evaluerar TFORM_MOVE
*    evtrrp();      Evaluerar TFORM_ROTP
*    evtrrl();      Evaluerar TFORM_ROTL
*    evtrmi();      Evaluerar TFORM_MIRR
*    evtrco();      Evaluerar TFORM_COMP
*    evtrus();      Evaluerar TFORM_USDEF
*    evtcop();      Evaluerar TCOPY
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"

/*!******************************************************/

        short evldim(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LDIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Execute LDIM.
*/
   return(EXldim( ident, (DBVector *)&pv[1].par_va.lit.vec_va, /* pos1 */
                         (DBVector *)&pv[2].par_va.lit.vec_va, /* pos2 */
                         (DBVector *)&pv[3].par_va.lit.vec_va, /* pos3 */
                   (short)pv[4].par_va.lit.int_va, /* alt */
                          npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evcdim(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CDIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Execute CDIM.
*/
   return(EXcdim( ident, &pv[1].par_va.lit.ref_va[0], /* ref */
                (DBVector *)&pv[2].par_va.lit.vec_va,    /* pos */
                   (short)pv[3].par_va.lit.int_va,    /* alt */
                          npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evrdim(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren RDIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Execute RDIM.
*/
   return(EXrdim( ident, &pv[1].par_va.lit.ref_va[0], /* ref */
                (DBVector *)&pv[2].par_va.lit.vec_va,    /* pos1 */
                (DBVector *)&pv[3].par_va.lit.vec_va,    /* pos2 */
                          npblockp));
 }

/********************************************************/
/*!******************************************************/

       short evadim(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ADIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Execute ADIM.
*/
   return(EXadim( ident, &pv[1].par_va.lit.ref_va[0], /* ref1 */
                         &pv[2].par_va.lit.ref_va[0], /* ref2 */
                (DBVector *)&pv[3].par_va.lit.vec_va,    /* pos */
                   (short)pv[4].par_va.lit.int_va,     /* alt */
                          npblockp));
 }

/********************************************************/
/*!******************************************************/

       short evxht(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp,
       short    npar)

/*      Evaluerar geometri-proceduren XHATCH.
 *
 *      In: id       => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          npar     => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
   short  nref = 0; /* Number of referenses */ 
   short  i;        /* loop variable */
   V2REFVA refarr[GMMXXH + 1]; /* reference array */

/*
***Skapa refarr.
*/
   for ( i = 3; i <= npar ; i++ )
     {
     refarr[ nref ].seq_val = pv[i].par_va.lit.ref_va[ 0 ].seq_val;
     refarr[ nref ].ord_val = pv[i].par_va.lit.ref_va[ 0 ].ord_val;
     refarr[ nref ].p_nextre = pv[i].par_va.lit.ref_va[ 0 ].p_nextre;
     nref++;
    }
/*
***Execute XHATCH.
*/
   return(EXxht( ident, refarr, nref,
                        pv[1].par_va.lit.float_va,
                        pv[2].par_va.lit.float_va,
                        npblockp));
 }

/********************************************************/
/*!******************************************************/

       short evarea(
       PMPARVA  pv[],
       PMLITVA *valp,
       short    npar)

/*      Evaluerar geometri-funktionen AREA.
 *
 *      In: pv       => Pekare till array med parametervrden.
 *          valp     => Pekare till resultat.
 *          npar     => Antal parametrar.
 *
 *      Ut: *valp = Berknad area.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 23/7/90 J. Kjellander
 *
 ******************************************************!*/

{
   short  nref = 0; /* Number of referenses */ 
   short  i;        /* loop variable */
   short  status;
   gmflt  area;
   DBVector  tp;
   V2REFVA refarr[GMMXXH + 1]; /* reference array */

/*
***Skapa refarr.
*/
   for ( i = 2; i <= npar ; i++ )
     {
     refarr[ nref ].seq_val = pv[i].par_va.lit.ref_va[ 0 ].seq_val;
     refarr[ nref ].ord_val = pv[i].par_va.lit.ref_va[ 0 ].ord_val;
     refarr[ nref ].p_nextre = pv[i].par_va.lit.ref_va[ 0 ].p_nextre;
     nref++;
    }
/*
***Execute.
*/
   status = EXarea(refarr,nref,pv[1].par_va.lit.float_va,&area,&tp);
   valp->lit.float_va = area;

   return(status);
 }

/********************************************************/
/*!******************************************************/

       short evsuar(
       PMPARVA  pv[],
       PMLITVA *valp)

/*      Evaluerar geometri-funktionen SURFACE_AREA.
 *
 *      In: pv       => Pekare till array med parametervrden.
 *          valp     => Pekare till resultat.
 *
 *      Ut: *valp = Berknad area.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-12-17 J. Kjellander
 *
 ******************************************************!*/

{
   short  status;
   gmflt  area;

/*
***Execute.
*/
   status = EXsuar(&pv[1].par_va.lit.ref_va[0],
                    pv[2].par_va.lit.float_va,
                   &area);

   valp->lit.float_va = area;
   
   return(status);
 }

/********************************************************/
/*!******************************************************/

       short evsear(
       PMPARVA  pv[],
       PMLITVA *valp)

/*      Evaluerar geometri-funktionen SECTION_AREA.
 *
 *      In: pv       => Pekare till array med parametervrden.
 *          valp     => Pekare till resultat.
 *
 *      Ut: *valp = Berknad area.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1997-12-17 J. Kjellander
 *
 ******************************************************!*/

{
   short  status;
   gmflt  area;

/*
***Execute.
*/
   status = EXsear(&pv[1].par_va.lit.ref_va[0],
                    pv[2].par_va.lit.float_va,
                   &area);

   valp->lit.float_va = area;
   
   return(status);
 }

/********************************************************/
/*!******************************************************/

       short evsecg(
       PMPARVA  pv[],
       PMLITVA *valp)

/*      Evaluerar geometri-funktionen SECTION_CGRAV.
 *
 *      In: pv       => Pekare till array med parametervrden.
 *          valp     => Pekare till resultat.
 *
 *      Ut: *valp = Berknad tyngdpunkt.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 2000-03-31 J. Kjellander
 *
 ******************************************************!*/

{
   short    status;
   DBVector cgrav;

/*
***Execute.
*/
   status = EXsecg(&pv[1].par_va.lit.ref_va[0],
                    pv[2].par_va.lit.float_va,
                   &cgrav);

   valp->lit.vec_va.x_val = cgrav.x_gm;
   valp->lit.vec_va.y_val = cgrav.y_gm;
   valp->lit.vec_va.z_val = cgrav.z_gm;
   
   return(status);
 }

/********************************************************/
/*!******************************************************/

       short evcgra(
       PMPARVA  pv[],
       PMLITVA *valp,
       short    npar)

/*      Evaluerar geometri-funktionen CGRAV.
 *
 *      In: pv       => Pekare till array med parametervrden.
 *          valp     => Pekare till resultat.
 *          npar     => Antal parametrar.
 *
 *      Ut: *valp = Berknad area.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 23/7/90 J. Kjellander
 *
 ******************************************************!*/

{
   short  nref = 0; /* Number of referenses */ 
   short  i;        /* loop variable */
   short  status;
   gmflt  area;
   DBVector  tp;
   V2REFVA refarr[GMMXXH + 1]; /* reference array */

/*
***Skapa refarr.
*/
   for ( i = 2; i <= npar ; i++ )
     {
     refarr[ nref ].seq_val = pv[i].par_va.lit.ref_va[ 0 ].seq_val;
     refarr[ nref ].ord_val = pv[i].par_va.lit.ref_va[ 0 ].ord_val;
     refarr[ nref ].p_nextre = pv[i].par_va.lit.ref_va[ 0 ].p_nextre;
     nref++;
    }
/*
***Execute.
*/
   status = EXarea(refarr,nref,pv[1].par_va.lit.float_va,&area,&tp);
   valp->lit.vec_va.x_val = tp.x_gm;
   valp->lit.vec_va.y_val = tp.y_gm;
   valp->lit.vec_va.z_val = 0.0;

   return(status);
 }

/********************************************************/
/*!******************************************************/

        short evsymb(
        PMREFVA *id,
        PMPARVA *pv,
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren SYMB.
 *
 *      In: id       => Symbolens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 14/3/86 J. Kjellander
 *
 ******************************************************!*/

  {
/*
***Exekvera.
*/
     return(EXsymb(id, pv[1].par_va.lit.str_va,
             (DBVector *)&pv[2].par_va.lit.vec_va,
                       pv[3].par_va.lit.float_va,
                       pv[4].par_va.lit.float_va,
                       npblockp));
  }

/********************************************************/
/*!******************************************************/

        short evtext(
        PMREFVA *ident,
        PMPARVA pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren TEXT.
 *
 *      In: ident    => Symbolens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera TEXT.
*/
   return(EXtext( ident,(DBVector *)&pv[1].par_va.lit.vec_va,
                                  pv[2].par_va.lit.float_va,
                                  pv[3].par_va.lit.str_va,
                                  npblockp));
}

/********************************************************/
/*!******************************************************/

        short evbpln(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren B_PLANE.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 27/8/87 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Execute B_PLANE.
*/
   return(EXbpln( ident, (DBVector *)&pv[1].par_va.lit.vec_va, /* pos1 */
                         (DBVector *)&pv[2].par_va.lit.vec_va, /* pos2 */
                         (DBVector *)&pv[3].par_va.lit.vec_va, /* pos3 */
                         (DBVector *)&pv[4].par_va.lit.vec_va, /* pos4 */
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrmv(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren TFORM_MOVE.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
    gmflt sx,sy,sz;

/*
***Har skalning begrts ?
*/
    if      ( np == 2 ) sx = sy = sz = 1.0;
    else if ( np == 3 ) sx = sy = sz = pv[3].par_va.lit.float_va;
    else if ( np == 4 )
      {
      sx = pv[3].par_va.lit.float_va;
      sy = sz = pv[4].par_va.lit.float_va;
      }
    else if ( np == 5 )
      {
      sx = pv[3].par_va.lit.float_va;
      sy = pv[4].par_va.lit.float_va;
      sz = pv[5].par_va.lit.float_va;
      }
/*
***Exekvera.
*/
    return(EXtrmv(ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                                   sx,sy,sz,
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrrp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren TFORM_ROTP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
    gmflt sx,sy,sz;
    DBVector p2;

/*
***Positionen P2 p rotationsaxeln = P1 + (0,0,1).
*/
    p2.x_gm = pv[1].par_va.lit.vec_va.x_val;
    p2.y_gm = pv[1].par_va.lit.vec_va.y_val;
    p2.z_gm = pv[1].par_va.lit.vec_va.z_val + 1.0;
/*
***Har skalning begrts ?
*/
    if      ( np == 2 ) sx = sy = sz = 1.0;
    else if ( np == 3 ) sx = sy = sz = pv[3].par_va.lit.float_va;
    else if ( np == 4 )
      {
      sx = pv[3].par_va.lit.float_va;
      sy = sz = pv[4].par_va.lit.float_va;
      }
    else if ( np == 5 )
      {
      sx = pv[3].par_va.lit.float_va;
      sy = pv[4].par_va.lit.float_va;
      sz = pv[5].par_va.lit.float_va;
      }
/*
***Exekvera.
*/
    return(EXtrro(ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                                  &p2,
                                   pv[2].par_va.lit.float_va,
                                   sx,sy,sz,
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrrl(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren TFORM_ROTL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
    gmflt sx,sy,sz;

/*
***Har skalning begrts ?
*/
    if      ( np == 3 ) sx = sy = sz = 1.0;
    else if ( np == 4 ) sx = sy = sz = pv[4].par_va.lit.float_va;
    else if ( np == 5 )
      {
      sx = pv[4].par_va.lit.float_va;
      sy = sz = pv[5].par_va.lit.float_va;
      }
    else if ( np == 6 )
      {
      sx = pv[4].par_va.lit.float_va;
      sy = pv[5].par_va.lit.float_va;
      sz = pv[6].par_va.lit.float_va;
      }
/*
***Exekvera.
*/
    return(EXtrro(ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                                   pv[3].par_va.lit.float_va,
                                   sx,sy,sz,
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrmi(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren TFORM_MIRR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
    DBVector p3;

/*
***Om bara 2 punkter getts skapar vi en tredje i Z-
***axelns riktning.
*/
    if ( np == 2 )
      {
      p3.x_gm = pv[1].par_va.lit.vec_va.x_val;
      p3.y_gm = pv[1].par_va.lit.vec_va.y_val;
      p3.z_gm = pv[1].par_va.lit.vec_va.z_val + 1.0;
      }
    else
      {
      p3.x_gm = pv[3].par_va.lit.vec_va.x_val;
      p3.y_gm = pv[3].par_va.lit.vec_va.y_val;
      p3.z_gm = pv[3].par_va.lit.vec_va.z_val;
      }
/*
***Exekvera.
*/
    return(EXtrmi(ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                                  &p3,
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrco(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren TFORM_COMP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/2/93 J. Kjellander
 *
 ******************************************************!*/

 {
    return(EXtrco(ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                         (DBVector *)&pv[3].par_va.lit.vec_va,
                         (DBVector *)&pv[4].par_va.lit.vec_va,
                         (DBVector *)&pv[5].par_va.lit.vec_va,
                         (DBVector *)&pv[6].par_va.lit.vec_va,
                                   npblockp));
 }

/********************************************************/
/*!******************************************************/

        short evtrus(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren TFORM_USDEF.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
   DBint   valadr;
   int     radsiz,fltsiz;
   PMLITVA fval;
   STTYTBL typtbl;
   STARRTY arrtyp;
   DBTmat  tr;

/*
***Berkna div. RTS-offset.
***radsiz = storleken p en FLOAT (4)       Normalt 32  bytes.
***fltsiz = storleken p en FLOAT           Normalt 8   bytes.
*/
   strtyp(pv[1].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&arrtyp);
   strtyp(arrtyp.base_arr,&typtbl);
   radsiz = typtbl.size_ty;

   strarr(typtbl.arr_ty,&arrtyp);
   strtyp(arrtyp.base_arr,&typtbl);
   fltsiz = typtbl.size_ty;
/*
***Kopiera 4X4-matrisen till DBTmat.
*/
   valadr = pv[1].par_va.lit.adr_va;

   ingval(valadr,arrtyp.base_arr,FALSE,&fval);
   tr.g11 = fval.lit.float_va;
   ingval(valadr+fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g12 = fval.lit.float_va;
   ingval(valadr+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g13 = fval.lit.float_va;
   ingval(valadr+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g14 = fval.lit.float_va;

   ingval(valadr+radsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g21 = fval.lit.float_va;
   ingval(valadr+radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g22 = fval.lit.float_va;
   ingval(valadr+radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g23 = fval.lit.float_va;
   ingval(valadr+radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g24 = fval.lit.float_va;

   ingval(valadr+2*radsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g31 = fval.lit.float_va;
   ingval(valadr+2*radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g32 = fval.lit.float_va;
   ingval(valadr+2*radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g33 = fval.lit.float_va;
   ingval(valadr+2*radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g34 = fval.lit.float_va;

   ingval(valadr+3*radsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g41 = fval.lit.float_va;
   ingval(valadr+3*radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g42 = fval.lit.float_va;
   ingval(valadr+3*radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g43 = fval.lit.float_va;
   ingval(valadr+3*radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
   tr.g44 = fval.lit.float_va;
/*
***Exekvera TFORM_USDEF.
*/
   return(EXtrus(ident,&tr,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evtcop(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren TCOPY.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 10/12/92 J. Kjellander
 *
 ******************************************************!*/

 {
    return(EXtcop(ident, &pv[1].par_va.lit.ref_va[0],
                         &pv[2].par_va.lit.ref_va[0],
                   (short)pv[3].par_va.lit.int_va,
                          npblockp));
 }

/********************************************************/
