/**********************************************************************
*
*    eval3.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evsrgm();     Evaluerar SET_ROOT_GM
*    evgngm();     Evaluerar GET_NEXT_GM
*    evgtid();     Evaluerar GETID
*    evgtyp();     Evaluerar GETTYP
*    evgint();     Evaluerar GETINT
*    evgflt();     Evaluerar GETFLT
*    evgstr();     Evaluerar GETSTR
*    evgvec();     Evaluerar GETVEC
*    evgref();     Evaluerar GETREF
*    evrstr();     Evaluerar RSTR
*    evrval();     Evaluerar RVAL
*    evival();     Evaluerar IVAL
*    evfval();     Evaluerar FVAL
*    evrefc();     Evaluerar REFC
*    evglor();     Evaluerar GLOBA_REF
*    evfins();     Evaluerar FINDS
*    evvecl();     Evaluerar VECL
*    evvecn();     Evaluerar VECN
*    evcrpr();     Evaluerar VPROD
*    evangl();     Evaluerar ANGLE
*    evgtts();     Evaluerar GET_TSTR
*    evgtal();     Evaluerar GET_ALT
*    evinpt();     Evaluerar INPMT
*    evinfn();     Evaluerar INFNAME
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"
#include "../../GE/include/GE.h"
#include <ctype.h>

extern short modtyp;

/*!******************************************************/

        short evsrgm(
        PMPARVA *pv,
        short    np)

/*      Evaluerar proceduren SET_ROOT_GM().
 *
 *      In: pv    => Pekare till array med parametervrden
 *          np    => antal parametrar.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1996-02-14 J. Kjellander
 *
 ******************************************************!*/

  {
    PMREFVA id;

/*
***Hr skulle det egentligen gtt med en optionell parameter
***med defaultvrde = #0.1 men systemet (formpar.h) klarar inte
***defaultvrden p parametrar av typen REF s vi fixar det s
***hr istllet.
*/
    if ( np == 1 )
      {
      return(DBset_root_id(pv[1].par_va.lit.ref_va));
      }
    else
      {
      id.seq_val = 0;
      id.ord_val = 1;
      id.p_nextre = NULL;
      return(DBset_root_id(&id));
      }
  }

/********************************************************/
/*!******************************************************/

        short evgngm(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GET_NEXT_GM().
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = ID.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1996-02-14 J. Kjellander
 *
 ******************************************************!*/

  {
   short   status;
   DBptr   la;
   char   *namn,*grupp;

/*
***Finns namn och/eller grupp med ? Isfall byter vi ut ev. ""
***mot NULL.
*/
   namn = pv[3].par_va.lit.str_va;
   if ( *namn == '*'  &&  *(namn+1) == '\0' ) namn = NULL;

   grupp = pv[4].par_va.lit.str_va;
   if ( *grupp == '*' &&  *(grupp+1) == '\0'  ) grupp = NULL;
/*
***Hmta LA till nsta storhet frn GM.
*/   
   status = DBget_next_ptr(pv[1].par_va.lit.int_va,
          (DBetype)pv[2].par_va.lit.int_va,
                   namn,
                   grupp,
                  &la);
   if ( status < 0 ) return(status);
/*
***verstt LA till ID. LA = DBNULL betyder slut p storheter.
*/
   if ( la == DBNULL )
     {
     valp->lit.ref_va[0].seq_val = 0;
     valp->lit.ref_va[0].ord_val = 1;
     valp->lit.ref_va[0].p_nextre = NULL;
     }
   else DBget_id(la,valp->lit.ref_va);
/*
***Slut.
*/
   return(0);
  }

/********************************************************/
/*!******************************************************/

        short evgtid(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETID.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = ID.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXgtid(&valp->lit.ref_va[0],pv[1].par_va.lit.str_va));
  }

/********************************************************/
/*!******************************************************/

        short evgtyp(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETTYP.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Typ.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {
    return(EXgtyp(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));
  }

/********************************************************/
/*!******************************************************/

        short evgint(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETINT.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Heltal.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXgint(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evgflt(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETFLT.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Flyttal.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXgflt(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evgstr(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETSTR.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXgstr(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evgvec(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETVEC.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Vector.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/3/86 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXgvec(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evgref(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GETREF.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Referens.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 29/5/86 R. Svedin
 *
 ******************************************************!*/

  {

    return(EXgref(&pv[1].par_va.lit.ref_va[0],
                    (short)pv[2].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evrstr(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen RSTR.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {

    return(EXrstr(&pv[1].par_va.lit.ref_va[0],
                  (short)pv[2].par_va.lit.int_va,
                  valp->lit.str_va));

  }

/********************************************************/
/*!******************************************************/

        short evrval(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen RVAL.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Referens.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 1/10/86 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXrval(pv[1].par_va.lit.str_va,&valp->lit.ref_va[0]));
  }

/********************************************************/
/*!******************************************************/

        short evival(
        PMPARVA *pv,
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen IVAL.
 *
 *      In: pv    = Pekare till array med parametervrden.
 *          valp  = Pekare till resultat.
 *          np    = Antal parametrar.
 *
 *      Ut: *valp = Heltal.
 *
 *      (C)microform ab 1998-09-04 J.Kjellander
 *
 ******************************************************!*/

  {
   int     status,n,tmp;
   char   *s;
   PMLITVA litval;

/*
***Prova att konvertera med sscanf().
*/
   if ( sscanf(pv[1].par_va.lit.str_va,"%d",&tmp) == 1 )
     {
     valp->lit.int_va = tmp;
     status = 0;
     }
   else
     {
     valp->lit.int_va = 0;
     status = -1;
     }
/*
***Returnera ev. status.
*/
   if ( np > 1 )
     {
     litval.lit.int_va = status;
     inwvar(pv[2].par_ty, pv[2].par_va.lit.adr_va,
             0, NULL, &litval);
     }
/*
***Returnera antal tecken.
*/
   if ( np > 2 )
     {
     n = 0;
     s = pv[1].par_va.lit.str_va;
     while ( *s == ' ' ) ++n, ++s;
     if ( *s == '-'  ||  *s == '+' ) ++n, ++s;
     while ( isdigit((int)*s) ) ++n, ++s;
     litval.lit.int_va = n;
     inwvar(pv[3].par_ty, pv[3].par_va.lit.adr_va,
             0, NULL, &litval);
     }
/*
***Slut.
*/
   return(0);
  }

/********************************************************/
/*!******************************************************/

        short evfval(
        PMPARVA *pv,
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen FVAL.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *          np    = Antal parametrar.
 *
 *      Ut: *valp = Flyttal.
 *
 *      (C)microform ab 1998-09-04 J.Kjellander
 *
 ******************************************************!*/

  {
   int     status,n;
   char   *s;
   PMLITVA litval;

/*
***Prova att konvertera med sscanf().
*/
   if ( sscanf(pv[1].par_va.lit.str_va,"%lf",&valp->lit.float_va) == 1 )
     status = 0;
   else
     {
     valp->lit.int_va = 0;
     status = -1;
     }
/*
***Returnera ev. status.
*/
   if ( np > 1 )
     {
     litval.lit.int_va = status;
     inwvar(pv[2].par_ty, pv[2].par_va.lit.adr_va,
             0, NULL, &litval);
     }
/*
***Returnera antal tecken.
*/
   if ( np > 2 )
     {
     n = 0;
     s = pv[1].par_va.lit.str_va;

     while ( *s == ' ' ) ++n, ++s;
     if ( *s == '-'  ||  *s == '+' ) ++n, ++s;
     if ( *s == '.' ) ++n, ++s;
     while ( isdigit((int)*s) ) ++n, ++s;
     if ( *s == '.' ) ++n, ++s;
     while ( isdigit((int)*s) ) ++n, ++s;
     if ( *s == 'e' ||  *s == 'E' )
       if ( *(s+1) == '-'  || *(s+1) == '+'  ||  isdigit(*(s+1)) ) n+=2, s+=2;
     while ( isdigit((int)*s) ) ++n, ++s;

     litval.lit.int_va = n;
     inwvar(pv[3].par_ty, pv[3].par_va.lit.adr_va,
             0, NULL, &litval);
     }
/*
***Slut.
*/
   return(0);
  }

/********************************************************/
/*!******************************************************/

        short evrefc(
        PMPARVA *pv,
        PMLITVA *valp,
        short   npar)

/*      Evaluerar funktionen REFC.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *          npar  => Antal parametrar.
 *
 *      Ut: *valp = Referens.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 7/10/86 R. Svedin
 *
 *      19/10/86 Bug, J. Kjellander
 *
 ******************************************************!*/

  {
    short pvi,vai;

/*
***Sekvens och ordningsnummer.
*/
    for ( pvi=1, vai=0; pvi <= npar; pvi += 2, vai++ )
      {
      valp->lit.ref_va[vai].seq_val = pv[pvi].par_va.lit.int_va;
      valp->lit.ref_va[vai].ord_val = pv[pvi+1].par_va.lit.int_va;
      }
/*
***Pekare.
*/
    valp->lit.ref_va[0].p_nextre = NULL;

    for ( vai=0; vai < ((npar+1)/2)-1; vai++ )
      {
      valp->lit.ref_va[vai].p_nextre = &valp->lit.ref_va[vai+1];
      valp->lit.ref_va[vai+1].p_nextre = NULL;
      }

    return(0);

  }

/********************************************************/
/*!******************************************************/

        short evglor(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GLOBAL_REF.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Referens.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 21/12/86 J. Kjellander
 *
 ******************************************************!*/

  {

    return(EXglor(&pv[1].par_va.lit.ref_va[0],
                   (short)pv[2].par_va.lit.int_va,
                  &valp->lit.ref_va[0]));

  }

/********************************************************/
/*!******************************************************/

        short evfins(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen FINDS.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Heltal.  
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 ******************************************************!*/

  {
    short status;
    short i;

    status=(EXfins(pv[1].par_va.lit.str_va,
                   pv[2].par_va.lit.str_va,&i));

    valp->lit.int_va = i;

    return(status);

  }

/********************************************************/
/*!******************************************************/

        short evvecl(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen VECL.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Flyttal. 
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 *      1999-03-03 GExxx, J.Kjellander
 *
 ******************************************************!*/

  {
    if ( modtyp == 2 ) valp->lit.float_va =
      GEvector_length2D((DBVector *)&pv[1].par_va.lit.vec_va);
    else valp->lit.float_va =
      GEvector_length3D((DBVector *)&pv[1].par_va.lit.vec_va);

    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evvecn(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen VECN.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Vektor.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 *      1999-03-03 GExxx, J.Kjellander
 *
 ******************************************************!*/

  {
    if ( modtyp == 2 )
      GEnormalise_vector2D((DBVector *)&pv[1].par_va.lit.vec_va,
                           (DBVector *)&valp->lit.vec_va);
    else
       GEnormalise_vector3D((DBVector *)&pv[1].par_va.lit.vec_va,
                            (DBVector *)&valp->lit.vec_va);
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evcrpr(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen VPROD.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp => Vektor.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 22/4/87  R. Svedin.
 *
 *      1999-03-03 GExxx, J.Kjellander
 *
 ******************************************************!*/

  {
    return(GEvector_product((DBVector *)&pv[1].par_va.lit.vec_va,
                            (DBVector *)&pv[2].par_va.lit.vec_va,
                            (DBVector *)&valp->lit.vec_va));
  }

/********************************************************/
/*!******************************************************/

        short evangl(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen ANGLE.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp => Flyttal.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 ******************************************************!*/

  {
    return(GExy_angled(pv[1].par_va.lit.float_va,
                       pv[2].par_va.lit.float_va,
                      &valp->lit.float_va));
  }

/********************************************************/
/*!******************************************************/

        short evgtts(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GET_TSTR.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 ******************************************************!*/

  {

    return(EXgtts((short)pv[1].par_va.lit.int_va,valp));

  }

/********************************************************/
/*!******************************************************/

        short evgtal(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen GET_ALT.
 *
 *      In: pv    => Pekare till array med parametervrden.
 *          valp  => Pekare till resultat.
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 13/4/87  R. Svedin.
 *
 ******************************************************!*/

  {

    return(EXgtal(pv[1].par_va.lit.str_va,valp));
  }

/********************************************************/
/*!******************************************************/

        short evinpt(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen INPMT.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 14/10/88 R. Svedin
 *
 ******************************************************!*/

  {
    return(EXinpt(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.str_va,
           (short)pv[3].par_va.lit.int_va,
                  valp->lit.str_va));

  }

/********************************************************/
/*!******************************************************/

        short evinfn(
        PMPARVA *pv,
        PMLITVA *valp)

/*      Evaluerar funktionen INFNAME.
 *
 *      In: pv    => Pekare till array med parametervrden
 *          valp  => Pekare till resultat
 *
 *      Ut: *valp = Strng.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1988-04-01 J.Kjellander
 *
 ******************************************************!*/

  {
    return(EXinfn(pv[1].par_va.lit.str_va,
                  pv[2].par_va.lit.str_va,
                  pv[3].par_va.lit.str_va,
                  pv[4].par_va.lit.str_va,
                  valp->lit.str_va));
  }

/********************************************************/
