/**********************************************************************
*
*    inac11.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    short evon()      evaluate on function 
*    short evend()     evaluate end function 
*    short evstrt()    evaluate start function 
*    short evion()     evaluate inv_on function
*    short evtang()    evaluate tang function
*    short evitan()    evaluate inv_tang function
*    short evcurv()    evaluate curv function
*    short evicur()    evaluate inv_curv function
*    short evarcl()    evaluate arcl function 
*    short eviarc()    evaluate inv_arcl function 
*    short evcent()    evaluate center function
*    short evnorm()    evaluate norm function
*    short evsect()    evaluate intersect function 
*    short evnsec()    evaluate n_intersect function 
*    short evvec()     evaluate vec function 
*    short evvecp()    evaluate vecp function
*    short evtxtl()    evaluate txtl function 
*    
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"
#include <stdlib.h>
#include <math.h>

#ifndef ANALYZER           /* don't compile if analyser */
extern DBTmat  *lsyspk;
#endif

extern short  modtyp;      /* indicate module type, from GM-part */

#ifndef ANALYZER           /* don't compile if analyser */

/*!******************************************************/

        short evrand(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    npar)

/*      Evaluate function RANDOM.
 *
 *      In:   pv[]  =>    Parameter value array
 *            npar  =>    Number of parameters
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int    range;
   double seed;

/*
***SCO/UNIX returnerar slumptal i intervallet frn och med 0
***till och med RAND_MAX = 2**15 - 1 = 32767.
***I LINUX r RAND_MAX = 2**31 - 1 = 2147483647. 1996-04-18 JK.
***VAX/VMS returnerar frn 0 till och med 2**31 - 1 = 2147483647.
*/
#ifdef UNIX
    range = RAND_MAX;
#endif

/*
***Samma fr WIN32.
*/
#ifdef WIN32
    range = RAND_MAX;
#endif

#ifdef VMS
    range = 2147483647;
#endif
/*
***Utan parameter anvnder vi C-funktionen rand().
***Med parameter anvnder vi srand(seed) som terstller
***slumptalsgeneratorn till det lge i serien som anges
***av seed.
*/
   if ( npar == 1 )
     {
     seed = pv[1].par_va.lit.float_va;
     if ( seed < 0.0  ||  seed > 1.0 ) return(erpush("IN5242",""));
     else srand((int)(seed*range));
     }

   valp->lit.float_va = (double)rand()/(double)range;

   return(0);
  }

/*!******************************************************/
/*!******************************************************/

        short evarcl(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function ARCL.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***execute ARCL , result type FLOAT
*/
   return(EXarcl( &pv[1].par_va.lit.ref_va[0],
                  &valp->lit.float_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short eviarc(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function INV_ARCL.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXiarc(&pv[1].par_va.lit.ref_va[0],
                  pv[2].par_va.lit.float_va,
                 &valp->lit.float_va));

  }  

/*!******************************************************/
/*!******************************************************/

        short evtxtl(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function TEXTL.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***execute TEXTL , result type FLOAT
*/
   return(EXtxtl( &pv[1].par_va.lit.str_va[0],
                  &valp->lit.float_va ));

  }  

/*!******************************************************/
/*!******************************************************/

        short evtang(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function TANG.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXtang(&pv[1].par_va.lit.ref_va[0],
                  pv[2].par_va.lit.float_va,
                  lsyspk, (DBVector *)&valp->lit.vec_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short evitan(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function INV_TANG.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXitan(&pv[1].par_va.lit.ref_va[0],
                (DBVector *)&pv[2].par_va.lit.vec_va,
                (short)pv[3].par_va.lit.int_va,
                 &valp->lit.float_va));

  }  

/*!******************************************************/
/*!******************************************************/

        short evcurv(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function CURV.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***execute CURV , result type FLOAT
*/
  return(EXcurv( &pv[1].par_va.lit.ref_va[0],
                  pv[2].par_va.lit.float_va,
                 &valp->lit.float_va ));

  }

/*!******************************************************/
/*!******************************************************/

        short evicur(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function INV_CURV.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXicur(&pv[1].par_va.lit.ref_va[0],
                  pv[2].par_va.lit.float_va,
           (short)pv[3].par_va.lit.int_va,
                 &valp->lit.float_va));

  }

/*!******************************************************/
/*!******************************************************/

        short evcen(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function CENTRE.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   short status;
/*
***execute CENTRE , result type VEC
*/
   if ( ( status = EXcen( &pv[1].par_va.lit.ref_va[0],
                           pv[2].par_va.lit.float_va,
                           lsyspk, (DBVector *)&valp->lit.vec_va ) ) != 0 )
          return( status );

   return( 0 );
  }

/*!******************************************************/
/*!******************************************************/

        short evnorm(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    npar)


/*      Evaluate function NORM.
 *
 *      In:   pv[]  =>    Parameter value array
 *            npar  =>    Number of parameters
 *        
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   gmflt u,v;

   if ( npar == 1 )
     {
     u = 0.0;
     v = 0.0;
     }
   else
     {
     switch ( pv[2].par_va.lit_type )
        {
        case C_INT_VA:
        u = (gmflt)pv[2].par_va.lit.int_va;
        v = 0.0;
        break;

        case C_FLO_VA:
        u = (gmflt)pv[2].par_va.lit.float_va;
        v = 0.0;
        break;

        case C_VEC_VA:
        u = (gmflt)pv[2].par_va.lit.vec_va.x_val;
        v = (gmflt)pv[2].par_va.lit.vec_va.y_val;
        break;
 
        default:
        return(erpush("IN5092","NORM"));
        }
     }

   return(EXnorm(&pv[1].par_va.lit.ref_va[0],u,v,(DBVector *)&valp->lit.vec_va)); 
  } 

/*!******************************************************/
/*!******************************************************/

        short evstrt(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function START.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXstrt(&pv[1].par_va.lit.ref_va[0],(DBVector *)&valp->lit.vec_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short evend(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function END.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXend(&pv[1].par_va.lit.ref_va[0],(DBVector *)&valp->lit.vec_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short evon(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    npar)

/*      Evaluate function ON.
 *
 *      In:   pv[]  =>    Parameter value array
 *            npar  =>    Number of parameters
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   gmflt u,v;

   if ( npar == 1 )
     {
     u = 0.0;
     v = 0.0;
     }
   else
     {
     switch ( pv[2].par_va.lit_type )
        {
        case C_INT_VA:
        u = (gmflt)pv[2].par_va.lit.int_va;
        v = 0.0;
        break;

        case C_FLO_VA:
        u = (gmflt)pv[2].par_va.lit.float_va;
        v = 0.0;
        break;

        case C_VEC_VA:
        u = (gmflt)pv[2].par_va.lit.vec_va.x_val;
        v = (gmflt)pv[2].par_va.lit.vec_va.y_val;
        break;
 
        default:
        return(erpush("IN5092","ON"));
        }
     }

   return(EXon(&pv[1].par_va.lit.ref_va[0],u,v,(DBVector *)&valp->lit.vec_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short evion(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function INV_ON.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXion(&pv[1].par_va.lit.ref_va[0],
               (DBVector *)&pv[2].par_va.lit.vec_va,
         (short)pv[3].par_va.lit.int_va,
               &valp->lit.float_va));
  }  

/*!******************************************************/
/*!******************************************************/

        short evsect(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function intersect .
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   return(EXsect(&pv[1].par_va.lit.ref_va[0],
                 &pv[2].par_va.lit.ref_va[0],
                  pv[3].par_va.lit.int_va,
                  pv[4].par_va.lit.int_va,
        (DBVector *)&valp->lit.vec_va));

  }

/*!******************************************************/
/*!******************************************************/

        short evnsec(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function n_intersect.
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   short numint,status;

   status = EXnsec(&pv[1].par_va.lit.ref_va[0],
                   &pv[2].par_va.lit.ref_va[0],
             (short)pv[3].par_va.lit.int_va,
                    lsyspk, &numint);

   if ( status < 0 ) return(status);

   valp->lit.int_va = (v2int)numint;
   return(0);
  }  

/*!******************************************************/

#endif

/*!******************************************************/

        short evvec(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function VEC.
 *
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***assign VEC , result type VEC
*/
   valp->lit.vec_va.x_val = pv[ 1 ].par_va.lit.float_va;
   valp->lit.vec_va.y_val = pv[ 2 ].par_va.lit.float_va;

   if ( modtyp == 2 )
       valp->lit.vec_va.z_val = 0.0;
   else
       valp->lit.vec_va.z_val = pv[ 3 ].par_va.lit.float_va;

   return( 0 );
  }  

/*!******************************************************/
/*!******************************************************/

        short evvecp(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function VECP.
 *
 *
 *      In:   pv[]  =>    Parameter value array
 *
 *      Out: *valp  =>    C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   v2float  radius;       /* parameters for VECP */
   v2float  angel;        /*      - " -          */
   v2float  zcomp;        /*      - " -          */

   radius = pv[ 1 ].par_va.lit.float_va;
   angel  = pv[ 2 ].par_va.lit.float_va;
   if ( modtyp == _2D )
       zcomp  = 0.0;
   else
       zcomp  = pv[ 3 ].par_va.lit.float_va;

/*
***calculate VECP , result type VEC
*/
   valp->lit.vec_va.x_val = radius * COS(angel*DGTORD);
   valp->lit.vec_va.y_val = radius * SIN(angel*DGTORD);
   valp->lit.vec_va.z_val = zcomp;

   return( 0 );
  }  

/*!******************************************************/
