/**********************************************************************
*
*    inac18.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    short evseek()           SEEK   - " -
*    short evouti()           OUTINT - " -
*    short evoutf()           OUTFLT - " -
*    short evouts()           OUTSTR - " -
*    short evoutb()           OUTBIN - " -
*    short evinio()           ININT function
*                             INFLT  - " -
*                             INSTR  - " -
*    short evinl()            INLIN  - " -
*    short evinb()            INBIN procedure
*    short evfpos()           FPOS function
*    short eviost()           IOSTAT function
*    short evufnm()           UNIQUE_FILENAME function
*
*    Internal routines:
*    short rfield()           read specified field from i/o
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../GP/include/GP.h"
#include "../include/indef.h"

#ifdef UNIX
#include <unistd.h>
#endif

static short rfield(V2FILE *f, int fieldlen, char *fieldstr);

/********************************************************/
/*!******************************************************/

        short evseek(
        PMPARVA pv[]) 

/*      Evaluate procedure SEEK.
 *
 *      In:    pv[]   =>  Parameter value array
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int fstatus;
   V2FILE *f;                /* parameter for SEEK */

/*
***FILE f 
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "u"
*/
   if ( !( ( f->mode[ 0 ] == 'u' ) && f->open ) )

/*
***trying to seek, when not open for "u"
*/
       { 
       f->iostat = -9;
       return(0);
       }

/*
***execute SEEK
*/
   if ( ( fstatus = fseek( f->fp, (long)pv[2].par_va.lit.int_va, 0 ) ) == 0 )
       f->iostat = 0;
   else
       f->iostat = -8;

   return( 0 );
  }  

/********************************************************/
/*!******************************************************/

        short evouti(
        PMPARVA pv[]) 

/*      Evaluate procedure OUTINT.
 *
 *      In:    pv[]   =>  Parameter value array
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2FILE *f;                  /* parameter for OUTINT */
   int format;                 /*       - " -          */

   char formstr[ IOFSTRL ];    /* the c:s format string */

/*
**FILE f
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "w", "u" or "a"
*/
   if ( !( ( (f->mode[0] == 'w') || (f->mode[0] == 'a') || (f->mode[0] == 'u') )
                                                         && f->open ) )
/*
***trying to write, when not open for "w", "u" or "a"
*/
       {
       f->iostat = -4;
       return(0);
       }

/*
***INT format, optional parameter
*/
   format = (int)pv[3].par_va.lit.int_va;

/*
***make c:s format string
*/
   mkform( C_INT_VA, format, (int)0, formstr );

/*
***Byt till ANSI-mode
*/
   if ( f->fp == stdout ) gpansi();  

/*
***execute OUTINT
*/
   fprintf( f->fp, formstr, (int)pv[2].par_va.lit.int_va );

/*
***flush std-output
*/
   if ( f->fp == stdout ) igflsh();    

   f->iostat = 0;

   return( 0 );
  }  

/********************************************************/
/*!******************************************************/

        short evoutf(
        PMPARVA pv[])

/*      Evaluate procedure OUTFLT.
 *
 *      In:    pv[]   =>  Parameter value array
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2FILE *f;                 /* parameter for OUTFLT  */
   int format1;               /*       - " -           */
   int format2;               /*       - " -           */

   char formstr[ IOFSTRL ];   /* the c:s format string */


/*
***FILE f
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "w", "u" or "a"
*/
   if ( !( ( (f->mode[0] == 'w') || (f->mode[0] == 'a') || (f->mode[0] == 'u') )
                                                      && f->open ) )
/*
***trying to write, when not open for "w", "u" or "a"
*/
       {
       f->iostat = -4;
       return(0);
       }

/*
***INT format1, optional parameter
*/
   format1 = (int)pv[3].par_va.lit.int_va;

/*
***INT format2, optional parameter 
*/
   format2 = (int)pv[4].par_va.lit.int_va;

/*
***make c:s format string
*/
   mkform( C_FLO_VA, format1, format2, formstr );

/*
***Byt till ANSI-mode
*/
   if ( f->fp == stdout ) gpansi();   

/*
***execute OUTFLT
*/
   fprintf( f->fp, formstr, pv[2].par_va.lit.float_va );

/*
***flush std-output
*/
   if ( f->fp == stdout ) igflsh();

   f->iostat = 0;

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evouts(
        PMPARVA pv[])     

/*      Evaluate procedure OUTSTR.
 *
 *      In:    pv[]   =>  Parameter value array
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2FILE *f;                     /* parameter for OUTSTR */
   char sdata[ V3STRLEN ];        /*       - " -          */
   int format;                    /*       - " -          */

   char formstr[ IOFSTRL ];       /* the c:s format string */

/*
***FILE f
*/
   f = pv[1].par_va.lit.fil_va;

/* 
***check if file is open for "w", "u" or "a"
*/
   if ( !( ( (f->mode[0] == 'w') || (f->mode[0] == 'a') || (f->mode[0] == 'u') )
                                                         && f->open ) )
/*
***trying to write, when not open for "w", "u" or "a"
*/
       {
       f->iostat = -4;
       return(0);
       }

/*
*** STRING sdata
*/
   strcpy( sdata, pv[2].par_va.lit.str_va );

/*
***INT format, optional parameter
*/
   format = (int)pv[3].par_va.lit.int_va;

/*
***check format
*/
   if ( format == 0 )
/*
***no actual parameter use string length
*/
       {
       format = strlen( sdata );
       } 

/*
***make c:s format string
*/
   mkform( C_STR_VA, format, (int)-1, formstr );

/*
***Byt till ANSI-mode
*/
   if ( f->fp == stdout ) gpansi();    

/*
***execute OUTSTR
*/
   fprintf( f->fp, formstr, sdata );

/*
***flush std-output
*/
   if ( f->fp == stdout ) igflsh();    

   f->iostat = 0;

   return( 0 );
  }  

/********************************************************/
/*!******************************************************/

        short evoutb(
        PMPARVA pv[])    

/*      Evaluate procedure OUTBIN.
 *
 *      In:    pv[]   =>  Parameter value array
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int fd;               /* file descriptor for low level I/O */
   long binsize;         /* size of the binary data block */
   char *binaddr=NULL;   /* address to the binary data block */
   STTYTBL type;         /* interface struct for type */

   V2FILE *f;            /* parameter for OUTBIN */


/*
***FILE f 
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "w", "u" or "a"
*/
   if ( !( ( (f->mode[0] == 'w') || (f->mode[0] == 'a') || (f->mode[0] == 'u') )
                                                         && f->open ) )
/*
***trying to write, when not open for "w", "u" or "a"
*/
       {
       f->iostat = -4;
       goto endl;
       }

/*
***bindata
***read type info for bindata
*/
   strtyp( pv[2].par_ty, &type );

/*
***check if the evaluated value is an address
*/
   if ( pv[2].par_va.lit_type == C_ADR_VA )
/*
***calculate run-time stack address
*/
       {
       binaddr = incrsa( pv[2].par_va.lit.adr_va );
       }
   else
       {
       switch( type.kind_ty )
         {
         case C_INT_VA:
             binaddr = (char *)&pv[2].par_va.lit.int_va;
             break;

         case C_FLO_VA:
             binaddr = (char *)&pv[2].par_va.lit.float_va;
             break;

         case C_STR_VA:
             binaddr = (char *)pv[2].par_va.lit.str_va;
             break;

         case C_VEC_VA:
             binaddr = (char *)&pv[2].par_va.lit.vec_va;
             break;

         case C_REF_VA:
             binaddr = (char *)pv[2].par_va.lit.ref_va;
             break;

         case C_FIL_VA:
             binaddr = (char *)pv[2].par_va.lit.fil_va;
             break;
         }
       }

   binsize = type.size_ty;

/*
***get the file descriptor
*/
   fd = fileno( f->fp );

/*
**execute OUTBIN
*/
   if ( write( fd, binaddr, binsize ) != binsize )
       {
       f->iostat = -6;
       }
   else
       f->iostat = 0;

   endl:

   return( 0 );

  }

/********************************************************/
/*!******************************************************/

        short evinio(
        pmvaty iotyp,
        PMPARVA pv[],
        PMLITVA *valp)

/*      Evaluate evaluate function IN-i/o.
 *
 *      In:   iotyp   =>  I/O data type 
 *            pv[]    =>  Parameter value array
 *
 *      Out:  *valp   =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int  fstatus;

   V2FILE *f;                    /* parameter for IN-i/o */
   int format,ival;              /*       - " -          */

   char fieldstr[ V3STRLEN ];    /* area for field under formated input */
   bool inform;                  /* indicates input format */
   char formstr[ IOFSTRL ];      /* the c:s format string */


/*
***FILE f
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "r" or "u"
*/
   if ( !( ((f->mode[0] == 'r') || (f->mode[0] == 'u')) && f->open ) )
/*
***trying to read, when not open for "r" or "u"
*/
       {
       f->iostat = -5;
       goto endl;
       }

/*
***INT format
*/
   format = (int)pv[2].par_va.lit.int_va;

/*
***check if no format
*/
   if ( format == 0 )
       {
       inform = FALSE; 
       } 
   else
       {
       inform = TRUE;
       }

/*
***make c:s format string
*/
   mkform( iotyp, format, (int)-2, formstr );

/*
***execute IN-i/o
*/
   if ( inform || f->fp == stdin )
/*
***Extract field and read from it
*/
       {
       rfield( f, format, fieldstr );
       switch( iotyp )
         {
         case C_INT_VA:
             fstatus = sscanf( fieldstr, formstr, &ival);
             valp->lit.int_va = ival;
             if ( f->iostat != EOF )
                 if ( fstatus == 0 )
                     f->iostat = -11;            /* item not read */
             break;
         case C_FLO_VA:
             fstatus = sscanf( fieldstr, formstr, &valp->lit.float_va );
             if ( f->iostat != EOF )
                 if ( fstatus == 0 )
                     f->iostat = -11;            /* item not read */
             break;
         case C_STR_VA:
             strcpy( valp->lit.str_va, fieldstr );
             break;
         }
       }
   else
       {
       switch( iotyp )
         {
         case C_INT_VA:
           if ( ( fstatus = fscanf( f->fp,formstr, &ival) ) == EOF )
/*
***End of file
*/
               {  
               f->iostat = -1;
               }
           else if ( fstatus == 0 )
               f->iostat = -11;                  /* item not read */
           else
               valp->lit.int_va = ival;
               f->iostat = 0;
           break;

         case C_FLO_VA:
           if ( ( fstatus = fscanf( f->fp,formstr, &valp->lit.float_va)) == EOF )
/*
***End of file
*/
               {
               f->iostat = -1;
               }
           else if ( fstatus == 0 )
            f->iostat = -11;                     /* item not read */
           else
               f->iostat = 0;
           break;

         case C_STR_VA:
           if ( ( fstatus = fscanf( f->fp,formstr, valp->lit.str_va ) ) == EOF )
/*
***End of file
*/
               { 
               f->iostat = -1;
               }
           else if ( fstatus == 0 )              /* Ny rad i samband med 1.9E JK */
               f->iostat = -11;                  /* item not read */
           else
               f->iostat = 0;
           break;
         }
       }

   endl:

   return( 0 );
  }  

/********************************************************/
/*!******************************************************/

        short evinl(
        PMPARVA pv[],
        PMLITVA *valp)

/*      Evaluate function INLIN.
 *
 *      In:   pv[]    =>  Parameter value array
 *
 *      Out:  *valp   =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   char *fschp;
   int slen;

   V2FILE *f;                 /* parameter for INLIN */

/*
***Ev. DEBUG.
*/
#ifdef DEBUG
   if ( dbglev(PMPAC) == 10 )
     {
     fprintf(dbgfil(PMPAC),"***Start-evinl***\n");
     fflush(dbgfil(PMPAC));
     }
#endif

/*
***FILE f 
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "r" or "u"
*/
   if ( !( ((f->mode[0] == 'r') || (f->mode[0] == 'u')) && f->open ) )
/*
***trying to read, when not open for "r" or "u"
*/
       {
       f->iostat = -5;
       return(0);
       }
/*
***Execute INLIN
***Om lsning frn terminal, ls med editering och eko.
*/
   if ( f->fp == stdin )
     {
     rfield(f,(int)V3STRLEN,valp->lit.str_va);
     }
/*
***Annars ls frn fil med fgets.
*/
   else
     {
     if ( ( fschp = fgets( valp->lit.str_va, V3STRLEN + 1, f->fp ) ) == NULL )
/*
***End of file or error
*/
       {
#ifdef DEBUG
    if ( dbglev(PMPAC) == 10 )
      {
      fprintf(dbgfil(PMPAC),"fgets() returnerar NULL !\n");
      fprintf(dbgfil(PMPAC),"ferror()=%d\n",ferror(f->fp));
      fprintf(dbgfil(PMPAC),"feof()=%d\n",feof(f->fp));
      fflush(dbgfil(PMPAC));
      }
#endif
       f->iostat = -1;
       }
     else
       {
       /* if newlin is in string, delete it */
       slen = strlen( valp->lit.str_va );
       if ( valp->lit.str_va[ slen -1 ] == '\n' )
           valp->lit.str_va[ slen - 1 ] = '\0';
#ifdef DEBUG
    if ( dbglev(PMPAC) == 10 )
      {
      fprintf(dbgfil(PMPAC),"fgets()=%s\n",valp->lit.str_va);
      fprintf(dbgfil(PMPAC),"ferror()=%d\n",ferror(f->fp));
      fprintf(dbgfil(PMPAC),"feof()=%d\n",feof(f->fp));
      fflush(dbgfil(PMPAC));
      }
#endif
       f->iostat = 0;
       }
     }
/*
***Ev. DEBUG.
*/
#ifdef DEBUG
    if ( dbglev(PMPAC) == 10 )
      {
      fprintf(dbgfil(PMPAC),"***Slut-evinl***\n\n");
      fflush(dbgfil(PMPAC));
      }
#endif

   return( 0 );

  }      

/********************************************************/
/*!******************************************************/

        short evinb(
        PMPARVA pv[],
        short   np)

/*      Evaluate function INBIN.
 *
 *      In:   pv[]    =>  Parameter value array
 *            np      =>  Number of parameters
 *
 *      Out:  
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int fd;             /* file descriptor for low level I/O */
   long binsize;       /* size of the binary data block */
   int  nbytes;        /* Optional user specified number of bytes */
   char *binaddr;      /* address to the binary data block */
   char  cdum;         /* Very short int */
   short sdum;         /* Short int */
   v2int idum;         /* VARKON int */
   float fdum;         /* C float */
   v2float ddum;       /* VARKON float */
   STTYTBL type;       /* type info */
   V2FILE *f;          /* parameter for INBIN */

/*
***FILE f 
*/
   f = pv[1].par_va.lit.fil_va;

/*
***check if file is open for "r" or "u"
*/
   if ( !( ((f->mode[0] == 'r') || (f->mode[0] == 'u') ) && f->open ) )
/*
***trying to write, when not open for "r" or "u"
*/
       {
       f->iostat = -5;
       return(0);
       }
/*
***check that the evaluated value is an address
*/
   if ( ! pv[2].par_va.lit_type == C_ADR_VA )
/*
***Not address, error
*/
       {
       return( erpush( "IN3263", "" ) );
       }
/*
***calculate run-time stack address
*/
   binaddr = incrsa( pv[2].par_va.lit.adr_va );

/*
***read bindata type info
*/
   strtyp( pv[2].par_ty, &type );

   binsize = type.size_ty;

/*
***get the file descriptor
*/
   fd = fileno( f->fp );

/*
***Ls s mnga bytes som variabeln r deklarerad till.
*/
   if ( np == 2 )
      {
      if ( read( fd, binaddr, binsize ) != binsize ) f->iostat = -7;
      else                                           f->iostat = 0;
      }
/*
***Ls av anvndaren specificerat antal bytes och konvertera.
*/
   else
      {
      nbytes = pv[3].par_va.lit.int_va;
      if ( nbytes < 1  ||  nbytes > 132 ) return(erpush("IN3422",""));

      switch( type.kind_ty )
        {
/*
***Fr INT r 1, 2 och 4 tilltet.
*/
        case C_INT_VA:
        if ( nbytes == sizeof(char) )
          {
          if ( read(fd,&cdum,nbytes) != nbytes ) f->iostat = -7;
          else
            {
            f->iostat = 0;
            idum = (v2int)cdum;
            V3MOME(&idum,binaddr,sizeof(idum));
            }
          }
        else if ( nbytes == sizeof(short) )
          {
          if ( read(fd,&sdum,nbytes) != nbytes ) f->iostat = -7;
          else
            {
            f->iostat = 0;
            idum = (v2int)sdum;
            V3MOME(&idum,binaddr,sizeof(idum));
            }
          }
        else if ( nbytes == sizeof(v2int) )
          {
          if ( read(fd,binaddr,nbytes) != nbytes ) f->iostat = -7;
          else                                     f->iostat =  0;
          }
        else return(erpush("IN3432",""));
        break;
/*
***Fr FLOAT r 4 och 8 tilltet.
*/
        case C_FLO_VA:
        if ( nbytes == sizeof(float) )
          {
          if ( read(fd,&fdum,nbytes) != nbytes ) f->iostat = -7;
          else
            {
            f->iostat = 0;
            ddum = (v2float)fdum;
            V3MOME(&ddum,binaddr,sizeof(ddum));
            }
          }
        else if ( nbytes == sizeof(v2float) )
          {
          if ( read(fd,binaddr,nbytes) != nbytes ) f->iostat = -7;
          else                                     f->iostat =  0;
          }
        else return(erpush("IN3432",""));
        break;
/*
***Fr STRING r 1 -> 132 tilltet.
*/
        case C_STR_VA:
        if ( read(fd,binaddr,nbytes) != nbytes ) f->iostat = -7;
        else                                     f->iostat =  0;
       *(binaddr+nbytes) = '\0';
        break;
 
        default:
        break;
        }
      }

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evfpos(
        PMPARVA pv[],
        PMLITVA *valp)    

/*      Evaluate function FPOS.
 *
 *      In:   pv[]    =>  Parameter value array
 *
 *      Out:  *valp   =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2FILE *f;             /* parameter for FPOS */

/*
***FILE f
*/
   f = pv[1].par_va.lit.fil_va;
/*
***check if file is open
*/
   if ( !( f->open ) )
/*
***trying to access, when not open
*/
       {
       f->iostat = -10;
       return(0);
       }
/* 
***Execute FPOS 
*/

/*
***Varfr skall fpos() returnera EOF nr man str i slutet ?
***P vaxen verkar det under inga frhllanden funka p rtt stt.
***l_anastr.MBS funkar inte p VAX:en. Har man en gng hamnat i slutet
***av filen verkar det som fpos() returnerar -1 fr all framtid ven
***om man med seek() flyttar sig tillbaks igen och ven om man sedan skriver
***och/eller lser. seek() verkar funka trots att fpos() hela tiden
***returnerar -1.
***Macro:t feof() i VAX-version returnerar inte samma resultat som
***i UNIX/DNIX/XENIX etc. ftell() verkar dock funka. Allts kan man strunta
***i om man fr EOF() i detta sammanhang.
*/
   if ( feof( f->fp ) )
       { 
#ifdef VAX
       f->iostat = 0;
       valp->lit.int_va = (v2int)ftell( f->fp );
#else
       f->iostat = -1;
       valp->lit.int_va = -1;
#endif
       }
else
       {
       valp->lit.int_va = (v2int)ftell( f->fp );
       f->iostat = 0;
       }

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short eviost(
        PMPARVA pv[],
        PMLITVA *valp)

/*      Evaluate function IOSTAT.
 *
 *      In:   pv[]    =>  Parameter value array
 *
 *      Out:  *valp   =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   V2FILE *f;              /* parameter for IOSTAT */

/*
***FILE f
*/
   f = pv[1].par_va.lit.fil_va;

/*
***Execute IOSTAT
*/
   valp->lit.int_va = (v2int)(f->iostat);

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evufnm(
        PMLITVA *valp)

/*      Evaluate function UNIQUE_FILENAME.
 *
 *      In:   
 *
 *      Out:  *valp   =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 199?-??-?? J, Kjellander
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
    char  template[7];

    strcpy(template,"XXXXXX");       /* Bugfix, 1999-03-01 JK */

    mktemp(template);

    strcpy(valp->lit.str_va,template);

    return(0);
  } 

/********************************************************/
/*!******************************************************/

        static short rfield(
        V2FILE *f,         
        int     fieldlen,  
        char   *fieldstr)

/*      Read field from i/o.
 *
 *      In:   *f        =>   Pointer to an MBS i/o controll block 
 *             fieldlen =>   The field length 
 *
 *      Out:  *f        =>   Pointer to an MBS i/o controll block 
 *            *fieldstr =>   out - the extracted field 
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-13 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   short ntkn;

/*
***Om lsning frn terminal, ls med editering och eko.
*/
   if ( f->fp == stdin )
     {
     if ( fieldlen == 0 ) ntkn = V3STRLEN;
     else ntkn = fieldlen;
     igglin("","",&ntkn,fieldstr);
     if ( ntkn == 0 )
       {
       f->iostat = -1;
       }
     else
       {
       f->iostat = 0;
       }
     }
/*
***Om lsning frn fil ls med fgets().
*/
   else
    {
     if ( fgets( fieldstr, fieldlen + 1, f->fp) == NULL )
/*
***End of file
*/
       {
       f->iostat = -1;
       }
     else
       f->iostat = 0;
    } 
   return( 0 );
  }

/********************************************************/
