/**********************************************************************
*
*    inproc.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    inevpr();     Interpreterar procedur
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../include/mbsrout.h"

/********************************************************/
/*!******************************************************/

        short inevpr(
        pm_ptr routla,
        pm_ptr arglist)

/*      Central rutin fr interpretering av ordinra
 *      procedurer.
 *
 *      In: routla  => Pekare till rutin.
 *          arglist => Pekare till aktuell parameterlista.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 14/3/86 J. Kjellander
 *
 ******************************************************!*/

 {
   STPROC rout;           /* Access structure for MBS routines */
   PMPARVA *pv;           /* pv, array for parameter values */
   short parcount;        /* number of actual parameters */
   short status;

/*
***Ls rutin-info frn symboltabell.
*/
   if ( ( status = strrou( routla, &rout ) ) < -1 )
        return( status );
/*
***Interpretera rutinens parameterlista. Special fr SET.
*/
   if ( rout.kind_pr != VSET  &&  rout.kind_pr != VSETB )
     {
     if ( ( status = inapar( arglist, routla, &rout, &pv, &parcount ) ) != 0 )
       {
       inrpar();
       return(status);
       }
     }
/*
***Exekvera proceduren.
*/

   switch ( rout.kind_pr )
     {
     case VSET:    /* OBS! special fr SET */
     if ( ( status = evset(arglist) ) < -1 ) return( status);
     else return(0);
     break;

     case VSETB:    /* OBS! special fr SET_BASIC */
     if ( ( status = evsetb(arglist) ) < -1 ) return( status);
     else return(0);
     break;

     case VMODLO:
     status = evmolo(pv);
     break;

     case VMODGL:
     status = evmogl();
     break;

     case VMODBA:
     status = evmoba();
     break;

     case VOS:
     status = evos(pv,parcount);
     break;

     case VOPEN:
     status = evopen(pv);
     break;

     case VSEEK:
     status = evseek(pv);
     break;

     case VOUTINT:
     status = evouti(pv);
     break;

     case VOUTFLT:
     status = evoutf(pv);
     break;

     case VOUTSTR:
     status = evouts(pv);
     break;

     case VOUTBIN:
     status = evoutb(pv);
     break;

     case VOUTLIN:
     status = evoutl(pv);
     break;

     case VINBIN:
     status = evinb(pv,parcount);
     break;

     case VCLOSE:
     status = evclos(pv,parcount);
     break;

     case VAPPFIL:
     status = evappf(pv);
     break;

     case VDELFIL:
     status = evdelf(pv);
     break;

     case VMOVFIL:
     status = evmovf(pv);
     break;

     case VCOPFIL:
     status = evcopf(pv);
     break;

     case VSRTGM:
     status = evsrgm(pv,parcount);
     break;

     case VGTHDR:
     status = evghdr(pv);
     break;

     case VGTPOI:
     status = evgpoi(pv);
     break;

     case VGTLIN:
     status = evglin(pv);
     break;

     case VGTARC:
     status = evgarc(pv);
     break;

     case VGTCUR:
     status = evgcur(pv);
     break;

     case VGTCUH:
     status = evgcuh(pv);
     break;

     case VGTSEG:
     status = evgseg(pv,parcount);
     break;

     case VGTSUH:
     status = evgsuh(pv);
     break;

     case VGTTOP:
     status = evgtop(pv);
     break;

     case VGTCUP:
     status = evgcub(pv);
     break;

     case VGTFAP:
     status = evgfac(pv);
     break;

     case VGTRAP:
     status = evgrat(pv);
     break;

     case VALLILS:
     status = evails(pv,parcount);
     break;

     case VALLILM:
     status = evailm(pv,parcount);
     break;

     case VALLIOS:
     status = evaios(pv,parcount);
     break;

     case VGTTRF:
     status = evgtrf(pv);
     break;

     case VGTTXT:
     status = evgtxt(pv);
     break;

     case VGTXHT:
     status = evgxht(pv);
     break;

     case VGTLDM:
     status = evgldm(pv);
     break;

     case VGTCDM:
     status = evgcdm(pv);
     break;

     case VGTRDM:
     status = evgrdm(pv);
     break;

     case VGTADM:
     status = evgadm(pv);
     break;

     case VGTGRP:
     status = evggrp(pv);
     break;

     case VGTCSY:
     status = evgcsy(pv);
     break;

     case VGTBPL:
     status = evgbpl(pv);
     break;

     case VGTPRT:
     status = evgprt(pv);
     break;

     case VUPHDR:
     status = evuhdr(pv);
     break;

     case VTRIM:
     status = evtrim(pv);
     break;

     case VDEL:
     status = evdel(pv);
     break;

     case VCLGM:
     status = evclgm(pv);
     break;

     case VCLPM:
     status = evclpm();
     break;

     case VLOADPM:
     status = evldpm(pv);
     break;

     case VBLNK:
     status = evblk(pv);
     break;

     case VUBLNK:
     status = evublk(pv);
     break;

     case VLDGM:
     status = evldgm(pv);
     break;

     case VSVGM:
     status = evsvgm(pv);
     break;

     case VLDJOB:
     status = evldjb(pv,parcount);
     break;

     case VSVJOB:
     status = evsvjb(pv);
     break;

     case VBLEV:
     status = evbllv(pv);
     break;

     case VUBLEV:
     status = evublv(pv);
     break;

     case VGTLEV:
     status = evgtlv(pv);
     break;

     case VGTVIEW:
     status = evgtvi(pv);
     break;

     case VCREVI:
     status = evcrvi(pv);
     break;

     case VACTVI:
     status = evacvi(pv);
     break;

     case VSCLVI:
     status = evscvi(pv);
     break;

     case VCENVI:
     status = evcevi(pv);
     break;

     case VHIDVI:
     status = evhdvi(pv,parcount);
     break;

     case VPRPVI:
     status = evprvi(pv);
     break;

     case VERAVI:
     status = evervi(pv);
     break;

     case VREPVI:
     status = evrpvi(pv,parcount);
     break;

     case VCACVI:
     status = evcavi(pv);
     break;

     case VPLTVI:
     status = evplvi(pv);
     break;

     case VLTVIEW:
     status = evltvi(pv,parcount);
     break;

     case VLTON:
     status = evlton(pv);
     break;

     case VLTOFF:
     status = evltof(pv);
     break;

     case VSHVIEW:
     status = evshvi(pv);
     break;

     case VMTVIEW:
     status = evmtvi(pv);
     break;

     case VCRCOL:
     status = evcrco(pv);
     break;

     case VPLTWI:
     status = evplwi(pv,parcount);
     break;

     case VGRID:
     status = evgrid(pv);
     break;

     case VGRIDX:
     status = evgrix(pv);
     break;

     case VGRIDY:
     status = evgriy(pv);
     break;

     case VGRIDDX:
     status = evgrdx(pv);
     break;

     case VGRIDDY:
     status = evgrdy(pv);
     break;

     case VMSIZE:
     status = evmsiz(pv);
     break;

     case VTIM:
     status = evtime(pv);
     break;

     case VLSIN:
     status = evlsin(pv);
     break;

     case VLSEX:
     status = evlsex(pv);
     break;

     case VLSLN:
     status = evlsln(pv);
     break;

     case VPSPMT:
     status = evpspt(pv);
     break;

     case VPOPMT:
     status = evpopt(pv);
     break;

     case VCRTSTR:
     status = evcrts(pv);
     break;

     case VCRMEN:
     status = evcrmy(pv,parcount);
     break;

     case VPSMEN:
     status = evpsmy(pv);
     break;

     case VPOMEN:
     status = evpomy(pv);
     break;

     case VGTMEN:
     status = evgtmy(pv);
     break;

     case VLDMDF:
     status = evlmdf(pv);
     break;

     case VSCR_2:
     status = evscr2(pv);
     break;

     case VPOS_2:
     status = evpos2(pv);
     break;

     case VIDNT_2:
     status = evidn2(pv);
     break;

     case VHIGHL:
     status = evhigh(pv,parcount);
     break;
/*
***Fnster-rutiner.
*/
     case VDLWIN:
     status = evdlwi(pv,parcount);
     break;

     case VSHWIN:
     status = evshwi(pv);
     break;

     case VGETWIN:
     status = evgtwi(pv,parcount);
     break;

     case VEXIT:
     status = evexit(pv,parcount);
     break;

     case VCLDLL:
     status = evcldl(pv,parcount);
     break;

     case VULDLL:
     status = evuldl(pv);
     break;

     case VCALEXT:
     status = evcaxt(pv);
     break;
/*
***MBS-rutiner.
*/
     case VRUNMBS:
     status = evrunm(pv);
     break;

     case VDELMBS:
     status = evdelm(pv);
     break;

     case VUPMBS:
     status = evupm(pv);
     break;

     case VUPPMBS:
     status = evuppm(pv);
     break;

     default:
     inrpar();                     /* release parameter pool space */
     return( erpush( "IN0104" , "" ) );
     break;
     }
/*
***Frigr parameter-pool utrymme.
*/
  inrpar();
/*
***Felkontroll.
*/
  if ( status == 0 ) return(0);
  else if ( status == -1 ) return(0);
  else return(status);
}

/********************************************************/

